import unittest
import sys
import os

current_dir = os.path.dirname(os.path.realpath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)
from Connection import MilvusConnection
from Collection import MilvusCollection
from Index import MilvusIndex
from pymilvus import FieldSchema, DataType

uri = "http://localhost:19530"
collectionName = "test_collection"
vectorName = "title_vector"
indexName = "vec_index"

milvusConnection = MilvusConnection()
collection = MilvusCollection()
milvusIndex = MilvusIndex()


class TestIndex(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        milvusConnection.connect(uri=uri)

        fields = [
            FieldSchema(name="id", dtype=DataType.VARCHAR, max_length=128),
            FieldSchema(name="title", dtype=DataType.VARCHAR, max_length=512),
            FieldSchema(name="title_vector", dtype=DataType.FLOAT_VECTOR, dim=768),
        ]
        collection.create_collection(
            collectionName=collectionName,
            fields=fields,
            autoId=False,
            description="this is a test collection",
            primaryField="id",
            isDynamic=True,
            consistencyLevel="Strong",
        )

    @classmethod
    def tearDownClass(cls):
        collection.drop_collection(collectionName)
        milvusConnection.disconnect()

    def test_create_index(self):
        params = [f"nlist:128"]
        res = milvusIndex.create_index(
            collectionName=collectionName,
            fieldName=vectorName,
            indexName=indexName,
            metricType="IP",
            indexType="IVF_SQ8",
            params=params,
        )
        self.assertEqual(res.code, 0)
        print(milvusIndex.list_indexes(collectionName))

    def test_describe_index(self):
        indexDetail = milvusIndex.get_index_details(collectionName, indexName)
        self.assertIn(indexName, indexDetail)

    def test_has_index(self):
        res = milvusIndex.has_index(collectionName, indexName)
        self.assertTrue(res)

    def test_drop_index(self):
        res = milvusIndex.drop_index(collectionName, vectorName)
        self.assertIsInstance(res, str)


if __name__ == "__main__":
    unittest.main()
